/* This file is part of the Allegria project
   Copyright (C) 2004 Francois Schiettekatte <francois.schiettekatte@umontreal.ca>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

// Widget to set detected element-related parameters that will be passed to erd2003

#include <qlabel.h>
#include <qtabwidget.h>
#include <qfiledialog.h>
#include <qmessagebox.h>

#include "ElementParam.h"
#include "ExperimentalSetupImpl.h"

ElementParamWidget::ElementParamWidget(ElementParam par, ExperimentalSetupImpl * pparent, const char* name ) 
		: FormElementParam( (QWidget *)pparent, name, true )
{

	parent = pparent;

	QDoubleValidator * dv = new QDoubleValidator(this);

	angle->setValidator(dv);
	ratio->setValidator(dv);
	thickness->setValidator(dv);
	slope->setValidator(dv);
	zero->setValidator(dv);
	Arho->setValidator(dv);

	detec->insertItem("ERD");
	detec->insertItem("RBS");

	setParams(par);

	connect(element, SIGNAL(clicked()), this, SLOT(setElement()));
	connect(absorber, SIGNAL(clicked()), this, SLOT(setAbsorber()));
	connect(detec, SIGNAL(activated(int)), this, SLOT(setCalib(int)));
	connect(findFile, SIGNAL(clicked()), this, SLOT(doFindFile()));

}


void ElementParamWidget::setElement() {

	detectedElement = parent->setElement(element, mass);
	setParams(parent->elemParam[detectedElement.Z]);

}

void ElementParamWidget::setAbsorber() {

	absorberElement = parent->setElement(absorber);

}

void ElementParamWidget::setParams(ElementParam par) {

	detectedElement = parent->findElement(par.atom.Z, element, par.atom.mass, mass);

    angle->setText(QString::number(par.angle));
	ratio->setText(QString::number(par.ratio));
	absorberElement = parent->findElement(par.absorber, absorber);
	thickness->setText(QString::number(par.thickness));

	slope->setText(QString::number(par.slope));
	zero->setText(QString::number(par.zero));
	Arho->setText(QString::number(par.Arho));

	if(par.detec == "RBS") {
		detec->setCurrentItem(1);
		slope->setEnabled(false);
		zero->setEnabled(false);
		Arho->setEnabled(false);

		// beam atom parameters
		ElementParam beamPar = parent->elemParam[parent->beamElement.Z];
		slope->setText(QString::number(beamPar.slope));
		zero->setText(QString::number(beamPar.zero));
		Arho->setText(QString::number(beamPar.Arho));

	} else {
		detec->setCurrentItem(0);
		slope->setEnabled(true);
		zero->setEnabled(true);
		Arho->setEnabled(true);
	}

	file->setText(par.file);

	parent->tabElementList->changeTab(this, par.atom.name);
}

ElementParam ElementParamWidget::getParams() {

	ElementParam par;

	par.atom = detectedElement;
	par.atom.mass = mass->currentText().toDouble();
	par.angle = angle->text().toDouble();
	par.ratio = ratio->text().toDouble();
	par.absorber = absorberElement.name;
	par.thickness = thickness->text().toDouble();
	par.slope = slope->text().toDouble();
	par.zero = zero->text().toDouble();
	par.Arho = Arho->text().toDouble();
	par.detec = detec->currentText();
	par.file = file->text();

	return par;
}


// if the ERD detection method is selected, apply detected atom calibration parameters
// while if RBS is selected, save the current parameters for this atom
// and apply beam atom calibration parameters
void ElementParamWidget::setCalib(int sel) {

	ElementParam currentPar = getParams();
	ElementParam par;

	if(detec->currentText()=="RBS") {

		// ERD->RBS: save the current parameters for this atom...
		parent->elemParam[detectedElement.Z] = getParams();

		// ... and get beam atom parameters
		par = parent->elemParam[parent->beamElement.Z];

	} else {

		// RBS->ERD: retrieve last parameters for this atom
		par = parent->elemParam[detectedElement.Z];

	}
	currentPar.slope = par.slope;
	currentPar.zero = par.zero;
	currentPar.Arho = par.Arho;

	setParams(currentPar);

}

void ElementParamWidget::doFindFile() {

   QString fileName( QFileDialog::getOpenFileName(parent->WorkingDirEdit->text()) );
   if ( !fileName.isNull() ) {
       QDir ds(fileName);
       QFileInfo fi(fileName);
       QDir dw(parent->WorkingDirEdit->text()+fi.fileName());

       if(ds!=dw) {
           if( QMessageBox::warning( this,
               "Allegria", "Spectrum files must be in the Working Directory for ERD to work. Use it anyway?",
               QMessageBox::Yes, QMessageBox::No) == QMessageBox::No)
               return;
       }
       file->setText(fi.fileName());
   }
}
