/* This file is part of the Allegria project
   Copyright (C) 2004 Francois Schiettekatte <francois.schiettekatte@umontreal.ca>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
//
// ExperimentalSetupImpl.cpp: implementation of the ExperimentalSetupImpl class.
//
// Tabbed Dialog to set the experimental parameters that will be passed to erd2003
// Also holds a tabbed ElementParamWidget for each detected element
//////////////////////////////////////////////////////////////////////

#include <qstring.h>
#include <qlabel.h>
#include <fstream.h>
#include <math.h>
#include <stdlib.h>
#include <qlineedit.h>
#include <qtabwidget.h>
#include <qspinbox.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qradiobutton.h>
#include <qtable.h>

#include "ExperimentalSetupImpl.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ExperimentalSetupImpl::ExperimentalSetupImpl(QWidget* parent, const char* name )
            : ExperimentalSetupDialog( parent, name, true )

{
    numElem = 0;
    ElementParamWidgetSize = FrameElem->frameGeometry().size();

    beamElement = Atom();
    monitorElement = Atom();

    loadParams();
    load("Allegria.exp"); // must be called after loadParams

    connect(BeamPushButton, SIGNAL(clicked()), this, SLOT(setBeam()));
    connect(MonitorPushButton, SIGNAL(clicked()), this, SLOT(setMonitor()));
    connect(buttonReadFromFile, SIGNAL(clicked()), this, SLOT(readFromFile()));
    connect(AddElemPushButton,SIGNAL(clicked()), this, SLOT(addElement()));
    connect(RemoveElemPushButton,SIGNAL(clicked()), this, SLOT(removeElement()));
    connect(WorkingDirPushButton,SIGNAL(clicked()), this, SLOT(selectDir()));
    connect(buttonSave,SIGNAL(clicked()), this, SLOT(saveParameterFilesAs()));
    connect(buttonOk,SIGNAL(clicked()), this, SLOT(saveParameterFiles()));

    connect(atRadioButton,SIGNAL(toggled(bool)), this, SLOT(setTextUnits(bool)));

    QDoubleValidator * dv = new QDoubleValidator(this);

    BeamEnergyEdit->setValidator(dv);
    BeamAngleEdit->setValidator(dv);
    LowerChannelEdit->setValidator(dv);
    StepSizeEdit->setValidator(dv);
    UpperChannelEdit->setValidator(dv);
    AlphaSlopeEdit->setValidator(dv);
    MinEnergyEdit->setValidator(dv);

}

ExperimentalSetupImpl::~ExperimentalSetupImpl()
{

}


// load data from an ERD parameter file
bool ExperimentalSetupImpl::load(QString filename)
{

    QString ss, ss1, ss2, ss3, ss4, ss5;
    double p1, p2, p3, p4;
    int i1;
    char s[1000], s1[1000], s2[1000], s3[1000], s4[1000], s5[1000];

    ifstream fi(filename);
    if(fi==NULL) return false;

    fi.getline(s,1000);
    ss = s;
    WorkingDirEdit->setText(ss);

    fi.getline(s,1000);
    ss=s;
    LogFileNameEdit->setText(ss);

    fi.getline(s,1000);
    ss=s;
    SampleDescEdit->setText(ss);

    // beam
    fi >> s >> p1 >> i1;  // beam name, mass, Z
    setBeam(i1, p1);

    fi >> s1 >> s2 >> s3 >> s4 >> s5; // beam angle, E0, Emin, step size, alpha_slope
    BeamAngleEdit->setText(s1);
    BeamEnergyEdit->setText(s2);
    MinEnergyEdit->setText(s3);
    StepSizeEdit->setText(s4);
    AlphaSlopeEdit->setText(s5);

    fi >> i1; // depth units: 1=1E16 at/cm2, 0=ug/cm2
    ugRadioButton->setChecked(i1==0);
    atRadioButton->setChecked(i1!=0);
    fi.getline(s,1000); // get to end of line

    // monitor
    fi >> s >> p1 >> i1 >> p2 >> p3; // name, m, Z, angle, ratio
    setMonitor(i1);
    fi >> s >> p1 >> p2 >> p4; // absorber, thickness, slope, zero
    fi >> p1 >> s >> s1 >> s2; // arho, method, ch_low, ch_high
    LowerChannelEdit->setText(s1);
    UpperChannelEdit->setText(s2);
    fi.getline(s,1000); // get to end of line

    // detected elements
    PeriodicTable pt;

    // create a new, empty tabElementList
    QRect tabGeom;
    if(tabElementList) {
        tabGeom = tabElementList->geometry();
        delete tabElementList;
    }
    tabElementList = new QTabWidget(tabElements);
    tabElementList->setGeometry(tabGeom);

    fi >> numElem;
    for(int i=0; i<numElem; i++) {
        ElementParam par;

        fi >> s >> p1 >> i1; // name, mass, Z

        par.atom = pt.returnAtom(i1);
        par.atom.mass = p1;

        fi >> par.angle >> par.ratio >> s >> par.thickness;
        par.absorber = s;

        fi >> p1 >> p2 >> p3 >> s; // slope, zero, arho, method
        par.detec = s;

        // copy the parameters only for ERD since, in the case of RBS
        // these are the incident beam values
        if(par.detec=="ERD") {
            par.slope = p1;
            par.zero = p2;
            par.Arho = p3;
        } else {
            // in the case of RBS, use previously saved values for the recoil
            par.slope = elemParam[par.atom.Z].slope;
            par.zero = elemParam[par.atom.Z].zero;
            par.Arho = elemParam[par.atom.Z].Arho;
        }

        fi >> s;
        par.file = s;

        fi.getline(s,1000); // get to end of line

        elementParamWidgetList[i] = new ElementParamWidget(par, this);
        elementParamWidgetList[i]->setFixedSize(ElementParamWidgetSize);
        tabElementList->addTab(elementParamWidgetList[i], par.atom.name);

    }

    fi >> i1;
    IterationSpinBox->setValue(i1);

    updateParams();
    return true;

}


void ExperimentalSetupImpl::setBeam() {

    beamElement = setElement(BeamPushButton, BeamMassComboBox);

}

void ExperimentalSetupImpl::setBeam(int Z, double mass) {

    beamElement = findElement(Z, BeamPushButton, mass, BeamMassComboBox);

}



void ExperimentalSetupImpl::setMonitor() {

    monitorElement = setElement(MonitorPushButton);

}

void ExperimentalSetupImpl::setMonitor(QString atomName) {

    monitorElement = findElement(atomName, MonitorPushButton);

}

void ExperimentalSetupImpl::setMonitor(int Z) {

    monitorElement = findElement(Z, MonitorPushButton);

}


Atom ExperimentalSetupImpl::setElement(QPushButton* pb, QComboBox * cb)
{
    Atom a = selectElement();
    pb->setText(a.name);

    if(cb!=NULL) {
        // set mass list, finding which place most abundant will occupy
        // (most abundant mass is in a.mass but not in a.isotopes)
        cb->clear();
        int select = 0;
        bool maiPlaced = false;
        for(int i=0; i<a.nIsotopes; i++) {
            if( !maiPlaced && a.isotopes[i]>a.mass ) {
                cb->insertItem(QString::number(a.mass));
                select = i;
                maiPlaced = true;
            }
            cb->insertItem(QString::number(a.isotopes[i]));
        }
        cb->setCurrentItem(select);
    }
    return a;

}


Atom ExperimentalSetupImpl::findElement(int atomNum, QPushButton* pb, double mass, QComboBox * cb)
{
    PeriodicTable periodicTable;
    Atom a = periodicTable.returnAtom(atomNum);
    
    pb->setText(a.name);

    if(cb!=NULL) {
        // set mass list
        // (most abundant mass is in a.mass but not in a.isotopes)
        bool maiPlaced = false;
        int i;
        cb->clear();

        for(i=0; i<a.nIsotopes; i++) {
            if( !maiPlaced && a.isotopes[i]>a.mass ) {
                cb->insertItem(QString::number(a.mass));
                maiPlaced = true;
            }
            cb->insertItem(QString::number(a.isotopes[i]));
        }

        // select mass closest to "mass" argument
        int select = 0;
        for(i=0; i<=a.nIsotopes; i++) {
            if(fabs(atof(cb->text(i))-mass)<fabs(atof(cb->text(select))-mass) )
                select = i;
        }
        cb->setCurrentItem(select);
    }
    return a;

}

Atom ExperimentalSetupImpl::findElement(QString atomName, QPushButton* pb) {

    PeriodicTable periodicTable;
    Atom a = periodicTable.returnAtom(atomName);

    pb->setText(a.name);

    return a;

}


Atom ExperimentalSetupImpl::selectElement()
{
    // Create a modal dialog window ...
    QDialog * periodicTableDialog = new QDialog(this, "periodicTableDialog", true);
    periodicTableDialog->setFixedSize(QSize(500,255));

    // ...that will containt a periodic table widget to select a single element
    PeriodicTable * periodicTable = new PeriodicTable(500,255, true, false, periodicTableDialog);

    // when the widget tels that an element is selected, close the dialog
    connect(periodicTable, SIGNAL(elementSelected()), periodicTableDialog, SLOT(accept()) );

    periodicTableDialog->exec();

    // return the selected element info
    return periodicTable->selectedElement;

}


void ExperimentalSetupImpl::readFromFile() {

    QString fileName( QFileDialog::getOpenFileName() );
    if ( fileName.isNull() )
        return;

    load(fileName);

}

bool ExperimentalSetupImpl::loadParams() {
    // load mass specufic parameters from file alegria.par
    char s[1000];
    int k;

    ifstream f("alegria.par");
    if( !f ) {
        QMessageBox::warning( this, "Allegria", "Error opening parameter file ALEGRIA.PAR","OK");
        return false;
    }

    PeriodicTable pt;

    f.getline(s,1000); // skip first line
    for(int i=1;i<MAXELEMENT; i++) {
        f >> k >> elemParam[i].angle >> elemParam[i].ratio >> s >> elemParam[i].thickness;
        elemParam[i].atom = pt.returnAtom(k);
        elemParam[i].absorber = s;

        f >> elemParam[i].slope >> elemParam[i].zero >> elemParam[i].Arho >> s;
        elemParam[i].detec = s;

        f.getline(s,1000);
        elemParam[i].file = QString(s).stripWhiteSpace();
    }

    f.close();
    return true;

}


// update elemParam from parameters in tabElementList tabs
// (as a side effect, if an element appears in more than one tab,
//  the params of its last occurence will be saved)
void ExperimentalSetupImpl::updateParams(){

    QStringList detecList = QStringList() << "ERD" << "RBS";

    for(int i=0; i<numElem; i++) {
        ElementParam par = elementParamWidgetList[i]->getParams();
        if(par.detec=="RBS") {
            // in this case, the current calibration parameters are those of the beam
            // and we don't want to save them as if they were those of this atom
            // so restore currently saved values for this atom
            par.slope = elemParam[par.atom.Z].slope;
            par.zero = elemParam[par.atom.Z].zero;
            par.Arho = elemParam[par.atom.Z].Arho;
        }
        // update
        elemParam[par.atom.Z] = par;
    }
}

bool ExperimentalSetupImpl::saveParameterFilesAs()
{
   QString fileName( QFileDialog::getSaveFileName(WorkingDirEdit->text()+QString("/Allegria.exp")) );
   if ( !fileName.isNull() )
       return saveParameterFiles(fileName);

   return true;
}

bool ExperimentalSetupImpl::saveParameterFiles( void )
{
    return saveParameterFiles("Allegria.exp");

}

bool ExperimentalSetupImpl::saveParameterFiles(QString filename)
{

    QString name;
    int i, mon = -1;

    // write an input file in the ERD2003.EXE format
    for(i=0; i<numElem; i++) {
        // find which selected element corresponds to the monitor
        ElementParam par = elementParamWidgetList[i]->getParams();
        if(par.atom.Z==monitorElement.Z) {
            mon = i;
            break;
        }

    }

    if(mon==-1) {
        // the monitor is not among the selected masses, we are in big troubles!!
        QMessageBox::warning( this, "AlegriaToF",
            "Monitor not found in selected masses...\n"
            "cannot generate parameter file",QMessageBox::Ok, 0);
        return false;
    }

    bool isRBS = false;
    updateParams();

        if(filename.isNull()) filename="Allegria.exp";

    ofstream fi(filename);

    fi << WorkingDirEdit->text() << "\n"
       << LogFileNameEdit->text() << "\n"
       << SampleDescEdit->text() << "\n";

    name = beamElement.name.upper();
    if(name.length()==1) name = name + "X";

    // beam
    fi << name.upper() << " "
       << BeamMassComboBox->currentText().toFloat() << " "
       << beamElement.Z << " "
       << BeamAngleEdit->text().toFloat() << " "
       << BeamEnergyEdit->text().toFloat() << " "
       << MinEnergyEdit->text().toFloat() << " "
       << StepSizeEdit->text().toFloat() << " "
       << AlphaSlopeEdit->text().toFloat() << " ";
    fi << atRadioButton->isChecked()?1:0;
    fi << "\n";

    // monitor
    ElementParam monitorPar = elementParamWidgetList[mon]->getParams();
    name = monitorElement.name.upper();
    if(name.length()==1) name = name + "X";

    bool b = monitorPar.detec==QString("RBS"); // when RBS, use beam slope, zero & Arho values
    if(b) isRBS = true;

    if(monitorPar.absorber.length()==1) monitorPar.absorber += "X";

    fi << name.upper() << " "
       << monitorElement.mass << " "
       << monitorElement.Z << " "
       << monitorPar.angle << " "
       << monitorPar.ratio << " "
       << monitorPar.absorber.upper() << " "
       << monitorPar.thickness << " ";
    if(b) {
        fi << elemParam[beamElement.Z].slope << " "
           << elemParam[beamElement.Z].zero  << " "
           << elemParam[beamElement.Z].Arho  << " ";
    } else {
        fi << monitorPar.slope << " "
           << monitorPar.zero << " "
           << monitorPar.Arho << " ";
    }

    fi << monitorPar.detec.upper() << " "
       << LowerChannelEdit->text().toInt() << " "
       << UpperChannelEdit->text().toInt() << " "
       << monitorPar.file.upper() << "\n";

    // detected elements
    fi << numElem << "\n";
    for(i=0; i<numElem; i++) {
        ElementParam par = elementParamWidgetList[i]->getParams();

        b = par.detec==QString("RBS"); // when RBS, use *beam* slope, zero & Arho values
        if(b) isRBS = true; // ... and tell the user we are doing so

        name = par.atom.name.upper();
        if(name.length()==1) name = name + "X";
        if(par.absorber.length()==1) par.absorber += "X";

        fi << name.upper() << " "
           << par.atom.mass << " "
           << par.atom.Z << " "
           << par.angle << " "
           << par.ratio << " "
           << par.absorber.upper() << " "
           << par.thickness << " ";
        if(b) {
            fi << elemParam[beamElement.Z].slope << " "
               << elemParam[beamElement.Z].zero  << " "
               << elemParam[beamElement.Z].Arho  << " ";
        } else {
            fi << par.slope << " "
               << par.zero << " "
               << par.Arho << " ";
        }

         fi << par.detec.upper() << " "
            << par.file.upper() << "\n";
    }

    fi << IterationSpinBox->text().toInt() << " XXXXX   0   0" << "\n";
    fi.close();

    if(isRBS) {
        QString message = QString("Beam parameters are used for RBS spectra.\n")
            + QString("slope = ") + QString::number(elemParam[beamElement.Z].slope)
            + QString(", zero = ")  + QString::number(elemParam[beamElement.Z].zero)
            + QString(", Arho = ")  + QString::number(elemParam[beamElement.Z].Arho);
        QMessageBox::information( this, "Alegria", message,"OK");
    }

    ofstream f("alegria.par");

    f << "Z angle ratio absorber thickness slope zero Arho detec file" << "\n";
    for(i=1;i<MAXELEMENT; i++) {
        f << i << " " << elemParam[i].angle << " "
          << elemParam[i].ratio  << " "
          << elemParam[i].absorber.upper() << " "
          << elemParam[i].thickness << " "
          << elemParam[i].slope << " "
          << elemParam[i].zero << " "
          << elemParam[i].Arho << " "
          << elemParam[i].detec.upper() << " "
          << elemParam[i].file.upper() << "\n";
    }

    f.close();

    return true;

}

void ExperimentalSetupImpl::addElement()
{
    elementParamWidgetList[numElem] = new ElementParamWidget(elemParam[1], this);
    elementParamWidgetList[numElem]->setFixedSize(ElementParamWidgetSize);
    tabElementList->addTab(elementParamWidgetList[numElem], elemParam[1].atom.name); // add H as default
    tabElementList->showPage(elementParamWidgetList[numElem]); 
    numElem++;
}

void ExperimentalSetupImpl::removeElement() 
{
    bool found = false;
    int j = 0;
    for(int i=0; i<numElem; i++) {
        if( !found && tabElementList->currentPage() == elementParamWidgetList[i] ) {
            tabElementList->removePage(elementParamWidgetList[i]);
            delete elementParamWidgetList[i];
            j++; // once the page to delete is found, page i will point to page i+1
            found = true;
            numElem--;
        }
        elementParamWidgetList[i] = elementParamWidgetList[j];
        j++;
    }
}


void ExperimentalSetupImpl::selectDir() {

    // ask for a directory
    QString dirName( QFileDialog::getExistingDirectory ( 
        WorkingDirEdit->text(), this, "savedir", "Select a working directory") );

    if ( !dirName.isNull() )
        WorkingDirEdit->setText(dirName);

}

void ExperimentalSetupImpl::setTextUnits(bool b) {

	if(b)TextLabel16->setText("step size\nx1E16 at/cm");
	else TextLabel16->setText("step size\n  g/cm");

}