/* This file is part of the Allegria project
   Copyright (C) 2004 Francois Schiettekatte <francois.schiettekatte@umontreal.ca>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

// This application window holds the differents graphs and periodic table, 
// and serves as an interface to the erd2003 program

#include <qaction.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qapplication.h>
#include <qworkspace.h>
#include <qmessagebox.h>
#include <qlayout.h>
#include <qprogressdialog.h>
#include <qspinbox.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>


#include <stdlib.h>
#include <fstream.h>
#include <math.h>

#include "allegriawindow.h"
#include "CalibView.h"
#include "../XYQt/colorseries.h"

// icones
#include "fileopen.xpm"
#include "filesave.xpm"
#include "expsetup.xpm"
#include "projection.xpm"
#include "Allegria.xpm"
#include "go.xpm"


#define ERDSIZE 1024

AllegriaWindow ::AllegriaWindow ( QWidget* parent, const char* name, WFlags f )
   : QMainWindow( parent, name, f )
{
   setCaption("Allegria");
   setIcon( QPixmap(allegria_xpm) );

   textbrowser = NULL;

   // read some working parameters saved after last run
   char s[1000];
   ifstream fi("allegria.ini");
   int w, h, xp, yp;  // window size and position

   if(fi) {
     fi.getline(s, 1000);
     workingdir = s;
     fi >> w >> h >> xp >> yp;
     fi.close();
   } else {
     QWidget *d = QApplication::desktop();
     xp=0; yp=0; w=d->width(); h=d->height(); // default size and position
     workingdir = QDir::currentDirPath(); // default dir
   }
   // graphs container (MDI management)
   QWorkspace* main = new QWorkspace(this);

   // menu setup
   QPopupMenu *file = new QPopupMenu( this );
   file->insertItem( "Save selected &energy spectrum as integer",  this, SLOT(saveEnergySpectrumAsInt()));
   file->insertItem( "Write energy &spectra in a single file",  this, SLOT(writeEnergySpectraMultiColumn()));
   file->insertItem( "&Read ERD depth profile",  this, SLOT(readDepthProfiles()));
   file->insertItem( "&Write depth profiles in a single file",  this, SLOT(writeDepthProfilesMultiColumn()));
   file->insertSeparator();
   file->insertItem( "E&xit",  qApp, SLOT(quit()), CTRL+Key_Q );
   file->insertItem( "&About", this, SLOT(about()) );

   QPopupMenu *convert = new QPopupMenu( this );
   convert->insertItem( "&Experimental parameters",  this, SLOT(experimentalSetup()), CTRL+Key_E );
   convert->insertItem( "&Go",  this, SLOT(go()), CTRL+Key_G );
   convert->insertItem( "&Show ERD output",  this, SLOT(showERDoutput()) );
   convert->insertItem( "Sur&face eneries",  this, SLOT(calibView()) );

   QPopupMenu *window = new QPopupMenu( this );
   window->insertItem( "&Tile",  main, SLOT(tile()), CTRL+SHIFT+Key_T );
   window->insertItem( "&Cascade",  main, SLOT(cascade ()), CTRL+SHIFT+Key_C );
   
   menu = new QMenuBar( this );
   menu->insertItem( "&File", file );
   menu->insertItem( "&Conversion", convert );
   menu->insertItem( "&Window", window );

   // toolbar
   QToolBar * tbar = new QToolBar("toolbar", this);
   new QToolButton(QPixmap(filesave), "Write depth profiles in single file", "Write depth profiles in single file", this, SLOT(writeDepthProfilesMultiColumn()), tbar );
   new QToolButton(QPixmap(expsetup_xpm), "Experimental parameters", "Experimental parameters", this, SLOT(experimentalSetup()), tbar );
   new QToolButton(QPixmap(go_xpm), "Start conversion", "Start conversion", this, SLOT(go()), tbar );


   energyGraph  = new FSGraph("ener_spec.agr", main);
   energyGraph->setGraphName("Mass separated energy spectra");

   profileGraph = new FSGraph("depth_profiles.agr", main);
   profileGraph->setGraphName("Depth profiles");

   setCentralWidget(main);
   resize(w,h);
   move(xp,yp);

   qApp->processEvents();     // process events so the main window shows up and we can tile its children
   main->tile();
}


AllegriaWindow ::~AllegriaWindow ()
{
   QPoint p = pos();
   QSize s = size();

   ofstream fo("allegria.ini");
   fo << workingdir << "\n";
   fo << s.width() << " " << s.height() << " " << p.x() << " " << p.y() << "\n";
   fo.close();

}



// load energy spectra, then start a conversion, and load the depth profiles
bool AllegriaWindow ::go()
{

   QString ss;
   QStringList proFiles; // profile file list
   QStringList elemList; // detected element list

   char s[1000], sElem[1000];

   // read directory and spectrum file names from parameter file
   ifstream fi("Allegria.exp");
   if(fi==NULL) return false;


   if(energyGraph->getndatasets() &&
     QMessageBox::warning( this, "Allegria",
                           "Clear energy spectrum graph?",
                            QMessageBox::Yes | QMessageBox::Default,
                            QMessageBox::No  | QMessageBox::Escape )==QMessageBox::Yes ) {

     energyGraph->clearDataSets();
   }


   fi.getline(s,1000);
   QString dataDir = s;
   if(dataDir.right(1)!="/") dataDir += "/";

   fi.getline(s,1000); // log file
   ERDoutput = dataDir + s;

   fi.getline(s,1000); // sample description
   fi.getline(s,1000); // beam data
   fi.getline(s,1000); // monitor

   // detected elements
   int i, n, iter;
   fi >> n;
   long *zz = new long[n];

   for(i=0; i<n; i++) {
     fi >> sElem >> s >> zz[i] >> s >> s; // name, m, Z, angle, ratio
     fi >> s >> s >> s >> s; // absorber, thickness, slope, zero
     fi >> s >> s >> s; // arho, method, filename

     QString S(s);
     QString specFile = dataDir + S;

     QFileInfo finfo(S);
     int ii = S.findRev(finfo.extension( FALSE ));
     if(ii>=0) S=S.left(ii-1); // filename with only the last extension chopped off

     proFiles << S + ".PRO"; // append profile file name...
     elemList << QString(sElem);   // ... and element name to list

     FSDataSet *sp = new FSDataSet(QString(sElem),QPen(colorSeries(i),1));
     if(!sp->loadfile(specFile)) {
       QMessageBox::warning( this, "Allegria", "Can't open file " + specFile, QMessageBox::Ok, 0);
       return false;
     }
     energyGraph->appendDataSet(sp);
     qApp->processEvents();     // process events so the datasets show up as they are loaded
   }
   fi >> iter;
   fi.close();

   // save parameters file into data directory
   ExperimentalSetupImpl * expSetup = new ExperimentalSetupImpl(this);
   expSetup->saveParameterFiles(dataDir+"Allegria.exp");
   delete expSetup;

   // start conversion
#ifdef _WS_WIN_
   system("erd2003.exe < Allegria.exp");
#else
   system("./erd2003.exe < Allegria.exp");
#endif

   // check conversion success
   int success;
   ifstream fcheck("PROGRESS.ERD");
   fcheck >> success;
   fcheck.close();
   if(success!=101) {
     QMessageBox::warning( this, "Allegria", "An error occured in ERD2003", QMessageBox::Ok, 0);
     return false;
   }


   // should we erase the profiles already present
   if(profileGraph->getndatasets() &&
     QMessageBox::warning( this, "Allegria",
                           "Clear profile graph?",
                            QMessageBox::Yes | QMessageBox::Default,
                            QMessageBox::No  | QMessageBox::Escape )==QMessageBox::Yes ) {

     profileGraph->clearDataSets();
   }



   // read back the profiles
   for(i=0; i<n; i++) {
     FSDataSet *sp = new FSDataSet(elemList[i],QPen(colorSeries(i),1));
     if(!sp->loadfile(dataDir+QString::number(iter)+proFiles[i])) {
       QMessageBox::warning( this, "Allegria", "Can't open file "
            + dataDir + QString::number(iter)+ proFiles[i], QMessageBox::Ok, 0);
     } else {
       for(int k=0; k<sp->getndata(); k++) sp->yp[k]/=10000.;
       profileGraph->appendDataSet(sp);
     }

   }

   delete zz;
   return true;

}


bool AllegriaWindow ::readDepthProfiles()
{

   // ask user for a file name
   QString fileName( QFileDialog::getOpenFileName() );
   if ( fileName.isNull() )
      return false;

   return readDepthProfiles(fileName, QPen(QColor(0,0,255),1));

}

bool AllegriaWindow ::readDepthProfiles(QString fileName, QPen pen, double stepSize )
{

   if ( !QFile::exists(fileName) )
      return false;

   //profileGraph->clearDataSets();
   FSDataSet* ds = new FSDataSet(fileName.right(7) , pen, 0, 1);

   ds->swallow(fileName);
   int nmax = ds->getndata();
   for(int n=0; n<nmax; n++) {
     ds->xp[n] = (ds->xp[n]-6.)*stepSize;     
     ds->yp[n]/=10000.;
   }
   profileGraph->appendDataSet(ds);
   profileGraph->update();

   return true;

}


void AllegriaWindow ::experimentalSetup()
{
   ExperimentalSetupImpl expSetup(this);

   // when user click OK, the dialog saves
   // experimental parameters in Allegria.exp
   expSetup.exec();

}


void AllegriaWindow ::writeEnergySpectraMultiColumn()
{
   if(!checkGraphData(energyGraph))
     return;

   QString fileName( QFileDialog::getSaveFileName() );
   if ( fileName.isNull() )
      return;

   ofstream fo(fileName);

   // ok, same x data, so lets ask a file name and save
   FSDataSet * ds = energyGraph->firstDataSet;
   int ndata = ds->getndata();

   fo << "energy";
   FSDataSet * dt = ds;
   while(dt != NULL) { 
     fo << " " << dt->getlegend();
     dt = dt->nextDataSet;
   }
   fo << "\n";

  // write the i_th spectrum value for each mass
   for(int i=0; i<ndata; i++) {
     fo << ds->xp[i];
   
     dt = ds;
     while(dt != NULL) { 
       fo << " " << dt->yp[i];
       dt = dt->nextDataSet;
     }
     fo << "\n";
   }
   fo.close();

}


void AllegriaWindow ::writeDepthProfilesMultiColumn()
{
   if(!checkGraphData(profileGraph))
     return;

   QString fileName( QFileDialog::getSaveFileName() );
   if ( fileName.isNull() )
      return;

   ofstream fo(fileName);

   // ok, same x data, so lets ask a file name and save
   FSDataSet * ds = profileGraph->firstDataSet;
   int ndata = ds->getndata();

   fo << "depth";
   FSDataSet * dt = ds;
   while(dt != NULL) { 
     fo << " " << dt->getlegend();
     dt = dt->nextDataSet;
   }
   fo << "\n";

  // write the i_th spectrum value for each mass
   for(int i=0; i<ndata; i++) {
     fo << ds->xp[i];

     dt = ds;
     while(dt != NULL) { 
       fo << " " << dt->yp[i];
       dt = dt->nextDataSet;
     }
     fo << "\n";
   }
   fo.close();

}

// return true if
bool AllegriaWindow ::checkGraphData(FSGraph * g)
{
   // check if datasets present
   int ndataset = g->getndatasets();
   if(ndataset==0) {
     QMessageBox::warning( this, "Allegria", "Graph contains no data.", QMessageBox::Ok, 0);
      return false;
   }

   // check if datasets have all the same set of x coordinates
   bool samex = true;
   double * x = g->firstDataSet->xp; // x coords of first datasets
   int ndata = g->firstDataSet->getndata();
   FSDataSet * ds = g->firstDataSet->nextDataSet;
   while(ds != NULL) { // check next dataset until there is no more
     if(ds->getndata()!=ndata) { // not the same number of data
       samex = false;
       break;
     }
     for(int i=0; i<ndata; i++) { // not the same x
       if(x[i]!=ds->xp[i]) {
         samex = false;
         break;
       }
     }
     if(!samex) break;
     ds = ds->nextDataSet;
   }
   if(!samex) {
     QMessageBox::warning( this, "Allegria", "Data must have same x coordinates.", QMessageBox::Ok, 0);
      return false;
   }

   return true;

}

void AllegriaWindow ::showERDoutput()
{

   char s[1000];
   QString ss;
   ifstream fi(ERDoutput);
   while(!fi.eof()) {
     fi.getline(s, 1000);
     ss = ss + s + "\n";
   }

   textbrowser = new QTextBrowser();
   textbrowser->setCaption("Allegria -- fort.11");
   textbrowser->resize(300,400);
   textbrowser->setText(ss);
   textbrowser->showNormal();

}


void AllegriaWindow::saveEnergySpectrumAsInt() {

   if(energyGraph->selectedDataSet) {
      QString fileName( QFileDialog::getSaveFileName() );
     if ( fileName.isNull() )
       return;

      energyGraph->selectedDataSet->savefile(fileName, true);
      energyGraph->updateGraph();
   }

}

void AllegriaWindow::calibView() {

   CalibView calibView(this);
   calibView.exec();
}

void AllegriaWindow::about() {

QMessageBox::about ( this, "About Allegria 2004", 
"Allegria (Allegria 2004)\n (c) 2004 Franois Schiettekatte, Universit de Montral\n"
"Released under GNU General Public Licence (www.gnu.org). This program cannot be sold.\n\n"
"If you use Allegria for published work, please include the following citation:\n"
"F. Schiettekatte, M. Chicoine, S. Gujrathi, P. Wei, K. Oxorn, Allegria: a new interface to the ERD program, in press, Nucl. Inst. Meth. (2004)\n\n"
"This program calls ERD2003, which uses SRIM 2000 stopping powers, so please also consider the following citation:\n"
"J.F. Ziegler, J.P. Biersack, U. Littmark , The Stopping and Range of Ions in Solids. Pergamon (New York) 1985");

}
