/* This file is part of the Allegria project
   Copyright (C) 2004 Francois Schiettekatte <francois.schiettekatte@umontreal.ca>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

// Dialog for editing axis properties

#include <qpen.h>
#include <qgrid.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvalidator.h>
#include <qcolordialog.h>
#include <qfontdialog.h>

#include <fstream.h>
#include "FSEditAxis.h"
#include "FSLineEditDouble.h"

FSEditAxis::FSEditAxis(FSAxis * axis, QWidget *parent, QString name )
            : QDialog( parent, name, true )
{

    selectedTitleFont = axis->titleFont;
    selectedLabelFont = axis->labelFont;
    selectedTitleColor = axis->titleColor;
    selectedLabelColor = axis->labelColor;
    pm.resize(66,20);

    QBoxLayout *vbox = new QVBoxLayout(this, 2, 2);

    // title
    QGroupBox *gbtitle = new QGroupBox( 1, QGroupBox::Horizontal, "title", this);
    vbox->addWidget(gbtitle);
    titleEdit = new QLineEdit( axis->title, gbtitle );

    QGrid *tabgrid = new QGrid( 3, this );
    vbox->addWidget(tabgrid);

    // title options
    QGroupBox *gbtitleOpt = new QGroupBox( 1, QGroupBox::Horizontal, "title options", tabgrid);
    btTitleFont = new QPushButton( "Font", gbtitleOpt );
    connect( btTitleFont, SIGNAL(clicked()), SLOT(newTitleFont()) );
    btTitleColor = new QPushButton( "Color", gbtitleOpt );
    pm.fill(selectedTitleColor);
    btTitleColor->setPixmap(pm);
    connect( btTitleColor, SIGNAL(clicked()), SLOT(newTitleColor()) );

    titleLeft = new QCheckBox( axis->isHorizontal?"bottom":"left",gbtitleOpt);
    titleLeft->setChecked(axis->gettitlePos()&1);
    titleRight = new QCheckBox( axis->isHorizontal?"top":"right",gbtitleOpt);
    titleRight->setChecked(axis->gettitlePos()&2);

    // labels
    QGroupBox *gblabel = new QGroupBox( 1, QGroupBox::Horizontal, "labels options", tabgrid);
    btLabelFont = new QPushButton( "Font", gblabel );
    connect( btLabelFont, SIGNAL(clicked()), SLOT(newLabelFont()) );
    btLabelColor = new QPushButton( "Color", gblabel );
    pm.fill(selectedLabelColor);
    btLabelColor->setPixmap(pm);
    connect( btLabelColor, SIGNAL(clicked()), SLOT(newLabelColor()) );

    labelLeft = new QCheckBox( axis->isHorizontal?"bottom":"left", gblabel);
    labelLeft->setChecked(axis->getlabelPos()&1);
    labelRight = new QCheckBox( axis->isHorizontal?"top":"right", gblabel);
    labelRight->setChecked(axis->getlabelPos()&2);

    // extrema
    QGroupBox *gbextrema = new QGroupBox( 1, QGroupBox::Horizontal, "extrema", tabgrid);
    QHBox *tabmin = new QHBox( gbextrema );
    (void)new QLabel( "min:", tabmin );
    minEdit = new FSLineEditDouble( QString::number(axis->logAxis?pow(10.,axis->getmin()):axis->getmin()), tabmin );
    minEdit->setFixedWidth(100);
    QHBox *tabmax = new QHBox( gbextrema );
    (void)new QLabel( "max:", tabmax );
    maxEdit = new FSLineEditDouble( QString::number(axis->logAxis?pow(10.,axis->getmax()):axis->getmax()), tabmax );
    maxEdit->setFixedWidth(100);

    // log axis
    (void)new QLabel( "", gbextrema );
    isLogEdit = new QCheckBox( "log axis",gbextrema);
    isLogEdit->setChecked(axis->logAxis);

    // frame pen and position
    QGroupBox *gbf = new QGroupBox( 1, QGroupBox::Horizontal, "frame", tabgrid);
    framePenEdit = new FSPenEdit(axis->framePen, "line options", 3, gbf);
    frameLeft = new QCheckBox( axis->isHorizontal?"bottom":"left", gbf);
    frameLeft->setChecked(axis->getframePos()&1);
    frameRight = new QCheckBox( axis->isHorizontal?"top":"right", gbf);
    frameRight->setChecked(axis->getframePos()&2);

    // tick pen and position
    QGroupBox *gbt = new QGroupBox( 1, QGroupBox::Horizontal, "tick marks", tabgrid);
    tickPenEdit = new FSPenEdit(axis->tickPen, "line options", 3, gbt);
    QHBox *tabntick = new QHBox( gbt );
    (void)new QLabel( "number:", tabntick );
    ntickEdit = new QLineEdit( QString::number(axis->getntick()), tabntick );
    ntickEdit->setFixedWidth(100);
    QIntValidator * validatorInt = new QIntValidator(2,99, this);
    ntickEdit->setValidator(validatorInt);
    tickInside = new QCheckBox( "inside", gbt);
    tickInside->setChecked(axis->gettickPos()&1);
    tickOutside = new QCheckBox( "outside", gbt);
    tickOutside->setChecked(axis->gettickPos()&2);

    // grid pen and position
    QGroupBox *gbg = new QGroupBox( 1, QGroupBox::Horizontal, "grid", tabgrid);
    gridPenEdit = new FSPenEdit(axis->gridPen, "line options", 3, gbg);
    drawGridEdit = new QCheckBox( "draw grid",gbg);
    drawGridEdit->setChecked(axis->grid);

    (void)new QLabel( "", tabgrid );

    ok = new QPushButton( "OK", tabgrid );
    ok->setDefault(true);
    connect( ok, SIGNAL(clicked()), SLOT(accept()) );

    cancel = new QPushButton( "Cancel", tabgrid );
    connect( cancel, SIGNAL(clicked()), SLOT(reject()) );

    setFixedSize(vbox->sizeHint());

}

QString FSEditAxis::title( void )
{
    return titleEdit->text();
}

int FSEditAxis::titlePos( void )
{
    int pos = 0;
    if(titleLeft->isChecked())pos+=1;
    if(titleRight->isChecked())pos+=2;
    return pos;
}

QFont FSEditAxis::titleFont( void )
{
    return selectedTitleFont;
}

QColor FSEditAxis::titleColor( void )
{
    return selectedTitleColor;
}


int FSEditAxis::labelPos( void )
{
    int pos = 0;
    if(labelLeft->isChecked())pos+=1;
    if(labelRight->isChecked())pos+=2;
    return pos;
}

QFont FSEditAxis::labelFont( void )
{
    return selectedLabelFont;
}

QColor FSEditAxis::labelColor( void )
{
    return selectedLabelColor;
}


QPen FSEditAxis::framePen( void )
{
    return framePenEdit->pen();
}

int FSEditAxis::framePos( void )
{
    int pos = 0;
    if(frameLeft->isChecked())pos+=1;
    if(frameRight->isChecked())pos+=2;
    return pos;
}


QPen FSEditAxis::tickPen( void )
{
    return tickPenEdit->pen();
}

int FSEditAxis::tickPos( void )
{
    int pos = 0;
    if(tickInside->isChecked())pos+=1;
    if(tickOutside->isChecked())pos+=2;
    return pos;
}

QPen FSEditAxis::gridPen( void )
{
    return gridPenEdit->pen();
}

bool FSEditAxis::drawGrid( void )
{
    return drawGridEdit->isChecked();
}


bool FSEditAxis::isLog( void )
{
    return isLogEdit->isChecked();
}

double FSEditAxis::min( void )
{
    return minEdit->text().toDouble();
}

double FSEditAxis::max( void )
{
    return maxEdit->text().toDouble();
}

bool FSEditAxis::extremaChanged( void )
{
   // min or max value were changed
    return (minEdit->edited() || maxEdit->edited());
}


int FSEditAxis::ntick( void )
{
    return ntickEdit->text().toInt();
}

void FSEditAxis::newTitleColor( void )
{
    QColor c = QColorDialog::getColor(selectedTitleColor, this);
    if(c.isValid())
    {
        selectedTitleColor = c;
        pm.fill(selectedTitleColor);
        btTitleColor->setPixmap(pm);
    }
}

void FSEditAxis::newLabelColor( void )
{
    QColor c = QColorDialog::getColor(selectedLabelColor, this);
    if(c.isValid())
    {
        selectedLabelColor = c;
        pm.fill(selectedLabelColor);
        btLabelColor->setPixmap(pm);
    }
}

void FSEditAxis::newTitleFont( void )
{
    bool ok;
    QFont f = QFontDialog::getFont(&ok, selectedTitleFont, this);
    if(ok)
        selectedTitleFont = f;
}

void FSEditAxis::newLabelFont( void )
{
    bool ok;
    QFont f = QFontDialog::getFont(&ok, selectedLabelFont, this);
    if(ok)
        selectedLabelFont = f;
}


